#!/usr/bin/python

# Copyright (C) 2013 Canonical
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; version 3.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import argparse
import imp
import importlib
import os
import sys

from gi.repository import Unity
from gi.repository import GLib

def load_scope(name, as_module):
  if as_module:
    # Treat as module name
    module = importlib.import_module(name)
  else:
    # Treat as source file name: use the file's base name as a module
    # name in the hope it is unique.
    modulename, _ = os.path.splitext(os.path.basename(name))
    module = imp.load_source(modulename, name)
  return module.load_scope()

def main(argv):
  GLib.threads_init()
  parser = argparse.ArgumentParser(
    description='A host process for Unity scopes written in Python')
  parser.add_argument(
    '-m', dest='as_module', action='store_true',
    help='Treat arguments as module names rather than file names')
  parser.add_argument(
    'names', metavar='names', nargs='+', type=str,
    help='The scopes to load')
  options = parser.parse_args(argv[1:])

  scopes = []
  connectors = []
  for name in options.names:
    scope = load_scope(name, options.as_module)
    connector = Unity.ScopeDBusConnector.new(scope)
    connector.export()
    scopes.append(scope)
    connectors.append(connector)
  # add manual handling for SIGTERM, since we don't use GLib.MainLoop
  # from python it wouldn't be handled properly
  GLib.unix_signal_add_full(0, 2, lambda x: Unity.ScopeDBusConnector.quit(), None)
  Unity.ScopeDBusConnector.run()

if __name__ == "__main__":
  main (sys.argv)

